@@ -1,10 +1,10 @@
 __title__ = 'efinance'
-__version__ = '0.3.3'
+__version__ = '0.3.9'
 __author__ = 'micro sheep'
 __url__ = 'https://github.com/Micro-sheep/efinance'
 __author_email__ = 'micro-sheep@outlook.com'
 __keywords__ = ['finance', 'quant', 'stock', 'fund', 'futures']
-__description__ = "A finance tool to get stock,fund and futures data base on eastmoney",
+__description__ = 'A finance tool to get stock,fund and futures data base on eastmoney'
 __project_urls__ = {
     'Documentation': 'https://efinance.readthedocs.io',
     'Source': 'https://github.com/Micro-sheep/efinance',
