@@ -1,3 +1,4 @@
+import rich
 import os
 import re
 from retry import retry
@@ -13,8 +14,10 @@ from jsonpath import jsonpath
 signal.signal(signal.SIGINT, multitasking.killall)
 
 
+@retry(tries=3)
 @to_numeric
-def get_quote_history(fund_code: str, pz: int = 40000) -> pd.DataFrame:
+def get_quote_history(fund_code: str,
+                      pz: int = 40000) -> pd.DataFrame:
     """
     根据基金代码和要获取的页码抓取基金净值信息
 
@@ -68,7 +71,9 @@ def get_quote_history(fund_code: str, pz: int = 40000) -> pd.DataFrame:
     }
     url = 'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNHisNetList'
     json_response = requests.get(
-        url, headers=EastmoneyFundHeaders, data=data).json()
+        url,
+        headers=EastmoneyFundHeaders,
+        data=data).json()
     rows = []
     columns = ['日期', '单位净值', '累计净值', '涨跌幅']
     if json_response is None:
@@ -85,16 +90,11 @@ def get_quote_history(fund_code: str, pz: int = 40000) -> pd.DataFrame:
             '累计净值': stock['LJJZ'],
             '涨跌幅': stock['JZZZL']
         })
-
     df = pd.DataFrame(rows)
-    df['单位净值'] = pd.to_numeric(df['单位净值'], errors='coerce')
-
-    df['累计净值'] = pd.to_numeric(df['累计净值'], errors='coerce')
-
-    df['日期'] = pd.to_datetime(df['日期'], errors='coerce')
     return df
 
 
+@retry(tries=3)
 @to_numeric
 def get_realtime_increase_rate(fund_codes: Union[List[str], str]) -> pd.DataFrame:
     """
@@ -109,18 +109,21 @@ def get_realtime_increase_rate(fund_codes: Union[List[str], str]) -> pd.DataFram
     -------
     DataFrame
         单只或者多只基金实时估算涨跌情况
+
     Examples
     --------
     >>> import efinance as ef
     >>> # 单只基金
     >>> ef.fund.get_realtime_increase_rate('161725')
-        基金代码              名称  估算涨跌幅              估算时间
-    0  161725  招商中证白酒指数(LOF)A  -0.64  2021-06-15 11:13
+        基金代码            基金名称    最新净值    最新净值公开日期              估算时间  估算涨跌幅
+    0  161725  招商中证白酒指数(LOF)A  2.8856  2021-09-07  2021-09-07 15:00   0.64
+
     >>> # 多只基金
     >>> ef.fund.get_realtime_increase_rate(['161725','005827'])
-        基金代码              名称  估算涨跌幅              估算时间
-    0  161725  招商中证白酒指数(LOF)A  -0.60  2021-06-15 11:16
-    1  005827       易方达蓝筹精选混合  -1.36  2021-06-15 11:16
+        基金代码            基金名称    最新净值    最新净值公开日期              估算时间  估算涨跌幅
+    0  161725  招商中证白酒指数(LOF)A  2.8856  2021-09-07  2021-09-07 15:00   0.64
+    1  005827       易方达蓝筹精选混合  2.5704  2021-09-07  2021-09-07 15:00   0.67
+
     """
 
     if not isinstance(fund_codes, list):
@@ -141,19 +144,25 @@ def get_realtime_increase_rate(fund_codes: Union[List[str], str]) -> pd.DataFram
     columns = {
         'FCODE': '基金代码',
         'SHORTNAME': '基金名称',
-        'GSZZL': '估算涨跌幅',
-        'GZTIME': '估算时间'
+        'ACCNAV': '最新净值',
+        'PDATE': '最新净值公开日期',
+        'GZTIME': '估算时间',
+        'GSZZL': '估算涨跌幅'
     }
+    url = 'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNFInfo'
     json_response = requests.get(
-        'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNFInfo', headers=EastmoneyFundHeaders, data=data).json()
+        url,
+        headers=EastmoneyFundHeaders,
+        data=data).json()
     rows = jsonpath(json_response, '$..Datas[:]')
     if not rows:
         df = pd.DataFrame(columns=columns.values())
         return df
-    df = pd.DataFrame(rows).rename(columns=columns)
+    df = pd.DataFrame(rows).rename(columns=columns)[columns.values()]
     return df
 
 
+@retry(tries=3)
 def get_fund_codes(ft: str = None) -> pd.DataFrame:
     """
     获取天天基金网公开的全部公墓基金名单
@@ -163,9 +172,14 @@ def get_fund_codes(ft: str = None) -> pd.DataFrame:
     ft : str, optional
         基金类型可选示例如下
 
-            - ``'zq'`` : 债券类型基金
-            - ``'gp'`` : 股票类型基金
-            - ``None`` : 全部
+        - ``'zq'``  : 债券类型基金
+        - ``'gp'``  : 股票类型基金
+        - ``'etf'`` : ETF 基金
+        - ``'hh'``  : 混合型基金
+        - ``'zs'``  : 指数型基金
+        - ``'fof'`` : FOF 基金
+        - ``'qdii'``: QDII 型基金
+        - ``None``  : 全部
 
     Returns
     -------
@@ -193,21 +207,19 @@ def get_fund_codes(ft: str = None) -> pd.DataFrame:
     1985  012516  国泰中证细分机械设备产业主题ETF联接A
 
     """
-
     params = [
-        ('op', 'ph'),
+        ('op', 'dy'),
         ('dt', 'kf'),
         ('rs', ''),
         ('gs', '0'),
-        ('sc', '6yzf'),
+        ('sc', 'qjzf'),
         ('st', 'desc'),
+        ('es', '0'),
         ('qdii', ''),
-        ('tabSubtype', ',,,,,'),
         ('pi', '1'),
         ('pn', '50000'),
-        ('dx', '1'),
-        ('v', '0.09350685300919159'),
-    ]
+        ('dx', '0')]
+
     headers = {
         'Connection': 'keep-alive',
         'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.141 Safari/537.36 Edg/87.0.664.75',
@@ -215,18 +227,22 @@ def get_fund_codes(ft: str = None) -> pd.DataFrame:
         'Referer': 'http://fund.eastmoney.com/data/fundranking.html',
         'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
     }
-    if ft is not None and ft in ['gp', 'zq']:
+    if ft is not None:
         params.append(('ft', ft))
 
+    url = 'http://fund.eastmoney.com/data/rankhandler.aspx'
     response = requests.get(
-        'http://fund.eastmoney.com/data/rankhandler.aspx', headers=headers, params=params)
-    results = re.findall('(\d{6}),(.*?),', response.text)
+        url,
+        headers=headers,
+        params=params)
+
     columns = ['基金代码', '基金简称']
-    results = re.findall('(\d{6}),(.*?),', response.text)
+    results = re.findall('"(\d{6}),(.*?),', response.text)
     df = pd.DataFrame(results, columns=columns)
     return df
 
 
+@retry(tries=3)
 @to_numeric
 def get_inverst_position(fund_code: str,
                          dates: Union[str, List[str]] = None) -> pd.DataFrame:
@@ -241,9 +257,9 @@ def get_inverst_position(fund_code: str,
         日期或者日期构成的列表
         可选示例如下
 
-            - ``None`` : 最新公开日期
-            - ``'2020-01-01'`` : 一个公开持仓日期
-            - ``['2020-12-31' ,'2019-12-31']`` : 多个公开持仓日期
+        - ``None`` : 最新公开日期
+        - ``'2020-01-01'`` : 一个公开持仓日期
+        - ``['2020-12-31' ,'2019-12-31']`` : 多个公开持仓日期
 
 
     Returns
@@ -317,8 +333,8 @@ def get_inverst_position(fund_code: str,
         ]
         if date is not None:
             params.append(('DATE', date))
-        params = tuple(params)
-        json_response = requests.get('https://fundmobapi.eastmoney.com/FundMNewApi/FundMNInverstPosition',
+        url = 'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNInverstPosition'
+        json_response = requests.get(url,
                                      headers=EastmoneyFundHeaders, params=params).json()
         stocks = jsonpath(json_response, '$..fundStocks[:]')
         if not stocks:
@@ -327,12 +343,13 @@ def get_inverst_position(fund_code: str,
         _df = pd.DataFrame(stocks)
         _df = _df.rename(columns=columns)
         _df['公开日期'] = [date for _ in range(len(_df))]
-        df = pd.concat([df, _df], axis=0)
+        df = pd.concat([df, _df], axis=0, ignore_index=True)
     df = df[columns.values()]
-    df.insert(0, '基金代码', [fund_code for _ in range(len(df))])
+    df.insert(0, '基金代码', fund_code)
     return df
 
 
+@retry(tries=3)
 @to_numeric
 def get_period_change(fund_code: str) -> pd.DataFrame:
     """
@@ -347,6 +364,7 @@ def get_period_change(fund_code: str) -> pd.DataFrame:
     -------
     DataFrame
         指定基金的阶段涨跌数据
+
     Examples
     --------
     >>> import efinance as ef
@@ -375,9 +393,11 @@ def get_period_change(fund_code: str) -> pd.DataFrame:
         ('product', 'EFund'),
         ('version', '6.3.6'),
     )
-
+    url = 'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNPeriodIncrease'
     json_response = requests.get(
-        'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNPeriodIncrease', headers=EastmoneyFundHeaders, params=params).json()
+        url,
+        headers=EastmoneyFundHeaders,
+        params=params).json()
     columns = {
 
         'syl': '收益率',
@@ -403,7 +423,7 @@ def get_period_change(fund_code: str) -> pd.DataFrame:
 
     df = df[list(columns.keys())].rename(columns=columns)
     df['时间段'] = titles.values()
-    df.insert(0, '基金代码', [fund_code for _ in range(len(df))])
+    df.insert(0, '基金代码', fund_code)
     return df
 
 
@@ -420,6 +440,7 @@ def get_public_dates(fund_code: str) -> List[str]:
     -------
     List[str]
         指定基金公开持仓的日期列表
+
     Examples
     --------
     >>> import efinance as ef
@@ -427,6 +448,7 @@ def get_public_dates(fund_code: str) -> List[str]:
     >>> # 展示前 5 个
     >>> public_dates[:5]
     ['2021-03-31', '2021-01-08', '2020-12-31', '2020-09-30', '2020-06-30']
+
     """
 
     params = (
@@ -439,14 +461,17 @@ def get_public_dates(fund_code: str) -> List[str]:
         ('serverVersion', '6.3.6'),
         ('version', '6.3.8'),
     )
-
+    url = 'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNIVInfoMultiple'
     json_response = requests.get(
-        'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNIVInfoMultiple', headers=EastmoneyFundHeaders, params=params).json()
+        url,
+        headers=EastmoneyFundHeaders,
+        params=params).json()
     if json_response['Datas'] is None:
         return []
     return json_response['Datas']
 
 
+@retry(tries=3)
 @to_numeric
 def get_types_persentage(fund_code: str,
                          dates: Union[List[str], str, None] = None) -> pd.DataFrame:
@@ -460,9 +485,9 @@ def get_types_persentage(fund_code: str,
     dates : Union[List[str], str, None]
         可选值类型示例如下(后面有获取 dates 的例子)
 
-            - ``None`` : 最新公开日期
-            - ``'2020-01-01'`` : 一个公开持仓日期
-            - ``['2020-12-31' ,'2019-12-31']`` : 多个公开持仓日期
+        - ``None`` : 最新公开日期
+        - ``'2020-01-01'`` : 一个公开持仓日期
+        - ``['2020-12-31' ,'2019-12-31']`` : 多个公开持仓日期
 
 
     Returns
@@ -510,18 +535,20 @@ def get_types_persentage(fund_code: str,
         if date is not None:
             params.append(('DATE', date))
         params = tuple(params)
+        url = 'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNAssetAllocationNew'
         json_response = requests.get(
-            'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNAssetAllocationNew',  params=params).json()
+            url,  params=params).json()
 
         if len(json_response['Datas']) == 0:
             continue
         _df = pd.DataFrame(json_response['Datas'])[columns.keys()]
         _df = _df.rename(columns=columns)
-        df = pd.concat([df, _df], axis=0)
-    df.insert(0, '基金代码', [fund_code for _ in range(len(df))])
+        df = pd.concat([df, _df], axis=0, ignore_index=True)
+    df.insert(0, '基金代码', fund_code)
     return df
 
 
+@retry(tries=3)
 @to_numeric
 def get_base_info_single(fund_code: str) -> pd.Series:
     """
@@ -545,9 +572,11 @@ def get_base_info_single(fund_code: str) -> pd.Series:
         ('product', 'EFund'),
         ('version', '6.3.8'),
     )
-
+    url = 'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNNBasicInformation'
     json_response = requests.get(
-        'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNNBasicInformation', headers=EastmoneyFundHeaders, params=params).json()
+        url,
+        headers=EastmoneyFundHeaders,
+        params=params).json()
     columns = {
         'FCODE': '基金代码',
         'SHORTNAME': '基金简称',
@@ -558,8 +587,14 @@ def get_base_info_single(fund_code: str) -> pd.Series:
         'FSRQ': '净值更新日期',
         'COMMENTS': '简介',
     }
+    items = json_response['Datas']
+    if not items:
+        rich.print('基金代码', fund_code, '可能有误')
+        return pd.Series(index=columns.values())
+
     s = pd.Series(json_response['Datas']).rename(
         index=columns)[columns.values()]
+
     s = s.apply(lambda x: x.replace('\n', ' ').strip()
                 if isinstance(x, str) else x)
     return s
@@ -587,9 +622,9 @@ def get_base_info_muliti(fund_codes: List[str]) -> pd.Series:
     def start(fund_code: str) -> None:
         s = get_base_info_single(fund_code)
         ss.append(s)
-        bar.update()
-        bar.set_description(f'processing {fund_code}')
-    bar = tqdm(total=len(fund_codes))
+        pbar.update()
+        pbar.set_description(f'Processing => {fund_code}')
+    pbar = tqdm(total=len(fund_codes))
     for fund_code in fund_codes:
         start(fund_code)
     multitasking.wait_for_tasks()
@@ -611,13 +646,14 @@ def get_base_info(fund_codes: Union[str, List[str]]) -> Union[pd.Series, pd.Data
     Union[Series, DataFrame]
         基金的一些基本信息
 
-            - ``Series`` : 包含单只基金基本信息(当 ``fund_codes`` 是字符串时)
-            - ``DataFrane`` : 包含多只股票基本信息(当 ``fund_codes`` 是字符串列表时)
+        - ``Series`` : 包含单只基金基本信息(当 ``fund_codes`` 是字符串时)
+        - ``DataFrane`` : 包含多只股票基本信息(当 ``fund_codes`` 是字符串列表时)
 
     Raises
     ------
     TypeError
         当 fund_codes 类型不符合要求时
+
     Examples
     --------
     >>> import efinance as ef
@@ -661,9 +697,9 @@ def get_industry_distribution(fund_code: str,
         日期
         可选示例如下
 
-            - ``None`` : 最新公开日期
-            - ``'2020-01-01'`` : 一个公开持仓日期
-            - ``['2020-12-31' ,'2019-12-31']`` : 多个公开持仓日期
+        - ``None`` : 最新公开日期
+        - ``'2020-01-01'`` : 一个公开持仓日期
+        - ``['2020-12-31' ,'2019-12-31']`` : 多个公开持仓日期
 
     Returns
     -------
@@ -727,15 +763,16 @@ def get_industry_distribution(fund_code: str,
         ]
         if date is not None:
             params.append(('DATE', date))
-
-        response = requests.get('https://fundmobapi.eastmoney.com/FundMNewApi/FundMNSectorAllocation',
-                                headers=EastmoneyFundHeaders, params=params)
+        url = 'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNSectorAllocation'
+        response = requests.get(url,
+                                headers=EastmoneyFundHeaders,
+                                params=params)
         datas = response.json()['Datas']
 
         _df = pd.DataFrame(datas)
         _df = _df.rename(columns=columns)
-        df = pd.concat([df, _df], axis=0)
-    df.insert(0, '基金代码', [fund_code for _ in range(len(df))])
+        df = pd.concat([df, _df], axis=0, ignore_index=True)
+    df.insert(0, '基金代码', fund_code)
     df = df.drop_duplicates()
     return df
 
@@ -754,6 +791,7 @@ def get_pdf_reports(fund_code: str,
         要获取的最大个数个 pdf (从最新的的开始数), 默认为 ``12``
     save_dir : str, optional
         pdf 保存的文件夹路径, 默认为 ``'pdf'``
+
     Examples
     --------
     >>> import efinance as ef
@@ -772,7 +810,10 @@ def get_pdf_reports(fund_code: str,
 
     @multitasking.task
     @retry(tries=3, delay=1)
-    def download_file(fund_code: str, url: str, filename: str, file_type='.pdf') -> None:
+    def download_file(fund_code: str,
+                      url: str,
+                      filename: str,
+                      file_type='.pdf') -> None:
         """
         根据文件名、文件直链等参数下载文件
 
@@ -788,7 +829,7 @@ def get_pdf_reports(fund_code: str,
             文件类型, 默认为 '.pdf'
         """
 
-        bar.set_description(f'processing {fund_code}')
+        pbar.set_description(f'Processing => {fund_code}')
         fund_code = str(fund_code)
         if not os.path.exists(save_dir+'/'+fund_code):
             os.mkdir(save_dir+'/'+fund_code)
@@ -799,7 +840,7 @@ def get_pdf_reports(fund_code: str,
         if os.path.getsize(path) == 0:
             os.remove(path)
             return
-        bar.update(1)
+        pbar.update(1)
     params = (
         ('fundcode', fund_code),
         ('pageIndex', '1'),
@@ -812,7 +853,7 @@ def get_pdf_reports(fund_code: str,
 
     base_link = 'http://pdf.dfcfw.com/pdf/H2_{}_1.pdf'
 
-    bar = tqdm(total=min(max_count, len(json_response['Data'])))
+    pbar = tqdm(total=min(max_count, len(json_response['Data'])))
     if not os.path.exists(save_dir):
         os.mkdir(save_dir)
     for item in json_response['Data'][-max_count:]:
@@ -821,5 +862,5 @@ def get_pdf_reports(fund_code: str,
         download_url = base_link.format(item['ID'])
         download_file(fund_code, download_url, title)
     multitasking.wait_for_tasks()
-    bar.close()
+    pbar.close()
     print(f'{fund_code} 的 pdf 文件已存储到文件夹 {save_dir}/{fund_code} 中')
